#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from urllib.parse import ParseResult, urlparse
import yaml
import logging
import subprocess

from pathlib import Path
from tuxtrigger.inputvalidation import YamlValidator
from tuxtrigger.exceptions import TuxtriggerException

LOG = logging.getLogger("tuxtrigger")


def yaml_load(data):
    return yaml.load(data, Loader=yaml.FullLoader)


def yaml_file_read(file_path: Path):
    with file_path.open("r") as file:
        loaded_yaml = yaml_load(file)
    return loaded_yaml


def yaml_file_write(data_input, file_path: Path):
    with file_path.open("w") as writer:
        yaml.dump(data_input, writer, default_flow_style=False, sort_keys=False)


def create_repo_list(data_input):
    if data_input is None:
        raise TuxtriggerException("Data input is none")
    for item in data_input["repositories"]:
        yield YamlValidator(**item)


def compare_sha(url_key, branch_name, input_dict, data_yaml_file) -> bool:
    if data_yaml_file is None:
        LOG.warning("\t*** Data Input is none, not able to compare sha")
        LOG.debug(f'\t-> sha: {input_dict[branch_name]["sha"]}')
        return True
    if branch_name not in data_yaml_file[url_key]:
        LOG.warning("\t*** Branch not found in yaml file")
        LOG.debug(f"\t*** branch name: {branch_name}")
        return False
    if (
        not input_dict[branch_name]["sha"]
        == data_yaml_file[url_key][branch_name]["sha"]
    ):
        LOG.info(
            f'\t-> sha: {input_dict[branch_name]["sha"]} vs \
        previous sha {data_yaml_file[url_key][branch_name]["sha"]}'
        )
        return True
    LOG.info(f'\t-> sha: {input_dict[branch_name]["sha"]}')
    LOG.info("\t-> no changes")
    return False


def create_squad_project(url: ParseResult, branch_name: str, prefix=None) -> str:
    url_fragment = (url.path).split("/")
    url_fragment = list(filter(None, url_fragment))
    if prefix is None:
        return f'{(url_fragment[-1]).replace(".git","")}-{branch_name.replace("/","-")}'
    return f'{prefix}-{(url_fragment[-1]).replace(".git","")}-{branch_name.replace("/","-")}'


def run_lsremote(repository: str, branch: str) -> dict:
    value_dict = dict()
    git_result = subprocess.run(
        ["git", "ls-remote", repository, branch],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    splited_value = git_result.stdout.split()
    value_dict[branch] = {"sha": "", "ref": ""}
    if git_result.returncode != 0:
        return value_dict

    if splited_value:
        value_dict[branch] = {"sha": splited_value[0], "ref": splited_value[1]}

    return value_dict


def run_lsremote_regex(repository: str, regex: str) -> dict:
    value_dict = dict()  # type: dict[str, str]
    git_result = subprocess.run(
        ["git", "ls-remote", "--refs", repository, regex],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    if git_result.returncode != 0:
        return value_dict

    splited_value = git_result.stdout.split("\n")
    for value in splited_value:
        if len(value) > 1:
            temp_list = value.split("\t")
            branch_list = temp_list[1].split("/")
            branch = "/".join(branch_list[int(len(branch_list) / -2) :])  # noqa: E203
            value_dict[branch] = temp_list[0]
    return value_dict


def create_dynamic_configuration(yaml_data):
    if yaml_data is None:
        raise TuxtriggerException(
            "*** Not able to generate data - config file not read"
        )
    LOG.debug("* Generating configuration")
    for repo in yaml_data["repositories"]:
        if "branches" not in repo.keys():
            repo["branches"] = []
        LOG.debug(f'Checking repository - {repo.get("url")}')
        if "regex" in repo:
            LOG.info(
                f'* Regex value present in config - {repo["regex"]} , looking for branches'
            )
            git_branch_updated = run_lsremote_regex(repo.get("url"), repo.get("regex"))
            for new_branch in git_branch_updated:
                if new_branch in str(repo["branches"]):
                    LOG.info(f"** Branch {new_branch} in config file, skip the branch")
                    continue
                repo["branches"].append(
                    {
                        "name": new_branch,
                        "squad_project": create_squad_project(
                            urlparse(repo.get("url").rstrip("/")),
                            new_branch,
                            repo.get("squad_project_prefix"),
                        ),
                        "plan": repo.get("default_plan"),
                    }
                )
            LOG.info("* Dynamic Config Generated")
    LOG.debug(f"updated yaml_data: {yaml_data}")
    return yaml_data


def pre_tux_run(script_path, repo_url, branch, branch_value_dict, repo_changed):
    if script_path is None:
        LOG.info("** No script path provided")
        return
    if not repo_changed:
        LOG.info("** Repo not changed, Script not invoked")
        return
    new_sha = branch_value_dict[branch]["sha"]
    script_result = subprocess.run(
        [script_path, repo_url, branch, new_sha],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    if script_result.returncode != 0:
        LOG.warning("script was not invoked properly")
        raise TuxtriggerException(
            f"** Script was not invoked properly - {script_result.stderr}"
        )
    LOG.info(f"stdout: {script_result.stdout}")
    LOG.info(f"*** Script {script_path} performed astonishingly wonderful")



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import os
import shutil
import subprocess
import json
import logging
import tempfile
from tuxtrigger.exceptions import (
    SquadException,
    TuxtriggerException,
    TuxsuiteException,
)

from urllib.parse import urlparse
from tuxtrigger.request import get_session, requests_post


LOG = logging.getLogger("tuxtrigger")
TIMEOUT = 60

squad_config = {
    "plugins": "linux_log_parser,ltp",
    "wait_notif_timeout": "600",
    "notif_timeout": "28800",
    "force_finish_timeout": "False",
    "metadata_keys": "build-url,git_ref,git_describe,git_repo,kernel_version",
    "user_treshold": "build/*-warnings",
    "data_retention": "0",
}


def tux_console_build(git_repo, git_ref, target_arch, kconfig, toolchain):
    with tempfile.NamedTemporaryFile(suffix=".json") as json_temp:
        build = subprocess.run(
            [
                "tuxsuite",
                "build",
                "--git-repo",
                git_repo,
                "--git-ref",
                git_ref,
                "--target-arch",
                target_arch,
                "--kconfig",
                kconfig,
                "--toolchain",
                toolchain,
                "--json-out",
                json_temp.name,
                "--no-wait",
            ],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            text=True,
        )
        if build.returncode != 0:
            LOG.warning(f"*build stdout {build.stdout}")
            LOG.warning(f"*build stderr {build.stderr}")
            raise TuxsuiteException(f"*** Tuxsuite not build repo {build.stderr}")

        LOG.debug(build.stdout)
        json_output = json.load(json_temp)
        LOG.debug(f'\t*** Build UID: {json_output["uid"]}')
        return json_output["uid"]


def tux_console_plan(
    json_data, plan_file, squad_group, squad_project, json_out, fingerprint
) -> int:
    if json_data is None:
        LOG.warning("\t** Not able to submit plan -> json output is None")
        raise TuxtriggerException("*** Not able to submit plan -> json output is None")
    with tempfile.NamedTemporaryFile(suffix=".json") as json_temp:
        plan = subprocess.run(
            [
                "tuxsuite",
                "plan",
                "--git-repo",
                json_data["git_repo"],
                "--git-ref",
                json_data["git_ref"],
                "--name",
                json_data["git_describe"],
                "--no-wait",
                plan_file,
                "--json-out",
                json_temp.name,
            ]
        )
        if plan.returncode != 0:
            LOG.warning(
                f'\t** Submiting Plan for {squad_project}_{json_data["git_describe"]} failed'
            )
            raise TuxsuiteException(
                f'*** Submiting Plan for {squad_project}_{json_data["git_describe"]} failed'
            )
        if json_out:
            try:
                shutil.copy(
                    json_temp.name,
                    json_out / f'{squad_project}_{json_data["git_describe"]}.json',
                )

                LOG.info(
                    f'\t*** Json file for git_describe: {json_data["git_describe"]} saved'
                )
            except TuxtriggerException as exc:
                LOG.warning(
                    f'*** Json file for git_describe: {json_data["git_describe"]} not saved: {str(exc)}'
                )
        LOG.info(f'\t-> Submiting Plan for {json_data["git_describe"]}')
        squad_submit(json_data, squad_group, squad_project, json_temp.name, fingerprint)
        return 0


def build_result(uid):
    if uid is None:
        return None
    build = subprocess.run(
        ["tuxsuite", "build", "get", uid, "--json"],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )
    if build.returncode != 0:
        LOG.warning(f"\t** Build result for UID:{uid} failed")
        raise TuxsuiteException(f"*** Build result for UID:{uid} failed")
    json_output = json.loads(build.stdout)
    LOG.debug(f"\t ** JSON OUTPUT: {json_output}")
    LOG.info(
        f'\t-> Build {json_output["uid"]} state: {json_output["state"]}, result: {json_output["result"]}, git describe: {json_output["git_describe"]}'
    )
    return json_output


def check_squad_project(squad_group, squad_project):
    base_url = os.getenv("SQUAD_HOST")
    session = get_session(retries=10)
    squad_response = session.get(
        url=f"{base_url}/api/projects/?slug={squad_project}&full_name={squad_group}",
        timeout=TIMEOUT,
    )
    if squad_response.status_code != 200:
        LOG.warning(
            f"*SQUAD response error - (get project id) {squad_response.status_code}"
        )
        raise SquadException(
            f"SQUAD response error - (get project id) {squad_response.status_code}"
        )
    json_output = json.loads(squad_response.content)
    if json_output["count"] != 0:
        LOG.info(
            f'*Project {squad_project} Exists!, parsing id {json_output["results"][0]["id"]}'
        )
        return json_output["results"][0]["id"]

    LOG.debug(f"*Project {squad_project} is empty or not created in SQUAD")
    LOG.info(f"*Creating Project - {squad_project}")
    squad_create = subprocess.run(
        [
            "squad-client",
            "create-or-update-project",
            "--group",
            squad_group,
            "--slug",
            squad_project,
            "--name",
            squad_project,
            "--is-public",
            "--plugins",
            squad_config["plugins"],
            "--wait-before-notification-timeout",
            squad_config["wait_notif_timeout"],
            "--notification-timeout",
            squad_config["notif_timeout"],
            "--force-finishing-builds-on-timeout",
            squad_config["force_finish_timeout"],
            "--important-metadata-keys",
            squad_config["metadata_keys"],
            "--threshold",
            squad_config["user_treshold"],
            "--data-retention",
            squad_config["data_retention"],
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )

    if squad_create.returncode != 0:
        LOG.warning(f"*build stdout {squad_create.stdout}")
        LOG.warning(f"*build stderr {squad_create.stderr}")
        raise SquadException(
            f"*** squad-client not able to create project {squad_create.stderr}"
        )

    squad_output = squad_create.stdout.split(" ")
    squad_url = urlparse(squad_output[-1])
    project_id = squad_url.path.split("/")
    return project_id[-2]


def update_squad_metadata(json_data, squad_group, squad_project, fingerprint):
    if squad_group is None or squad_project is None:
        LOG.warning("** SQUAD config is not available! Unable to process **")
        raise TuxtriggerException(
            "** SQUAD config is not available! Unable to process **"
        )

    headers = {"Auth-Token": os.getenv("SQUAD_TOKEN")}
    url = f'{os.getenv("SQUAD_HOST")}/api/submit/{squad_group}/{squad_project}/{json_data["git_describe"]}/{json_data["git_describe"]}'
    git_sha = json_data.get("git_sha", "")

    data = {"metadata": json.dumps({"fingerprint": fingerprint, "git_sha": git_sha})}
    result = requests_post(url=url, headers=headers, data=data)
    if not result.ok:
        LOG.warning(f"* Error submitting to SQUAD: {result.reason}: {result.text}")
        raise TuxtriggerException(
            f"Error submitting to SQUAD: {result.reason}: {result.text}"
        )
    LOG.info("* sha/fingerprint data submitted to SQUAD")
    return 0


def squad_submit(json_data, squad_group, squad_project, plan_json, fingerprint):
    if squad_group is None or squad_project is None:
        LOG.warning("** SQUAD config is not available! Unable to process **")
        raise TuxtriggerException(
            "** SQUAD config is not available! Unable to process **"
        )
    squad = subprocess.run(
        [
            "squad-client",
            "submit-tuxsuite",
            "--group",
            squad_group,
            "--project",
            squad_project,
            "--build",
            json_data["git_describe"],
            "--backend",
            "tuxsuite.com",
            "--json",
            plan_json,
        ],
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        text=True,
    )

    if squad.returncode != 0:
        LOG.warning(f"*build stdout {squad.stdout}")
        LOG.warning(f"*build stderr {squad.stderr}")
        raise SquadException(f"*** squad-client not able to pass data {squad.stderr}")

    LOG.debug(squad.stdout)
    LOG.info(f'\t-> Plan submitted to SQUAD git describe: {json_data["git_describe"]}')
    return 0


def squad_metadata_request(squad_project_id, squad_project):
    if squad_project_id is None or squad_project is None:
        LOG.debug(f"SQUAD project {squad_project}")
        LOG.warning("** SQUAD Project is not available!")
        return (None, None)

    base_url = os.getenv("SQUAD_HOST")
    session = get_session(retries=10)

    build_response = session.get(
        url=f"{base_url}/api/projects/{squad_project_id}/builds/?ordering=-datetime",
        timeout=TIMEOUT,
    )
    if build_response.status_code != 200:
        LOG.warning(
            f"*SQUAD response error - (get latest build) {build_response.status_code}"
        )
        raise SquadException(
            f"SQUAD response error - (get latest build) {build_response.status_code}"
        )
    build_json = json.loads(build_response.content)
    if build_json["count"] == 0:
        LOG.warning("**git sha not available in SQUAD metadata")
        return ("", "")

    lastest_version_url = build_json["results"][0]["metadata"]

    metadata_response = session.get(url=lastest_version_url, timeout=TIMEOUT)
    if metadata_response.status_code != 200:
        LOG.warning(
            f"*SQUAD response error - (get build metadata) {metadata_response.status_code}"
        )
        raise SquadException(
            f"SQUAD response error - (get build metadata) {metadata_response.status_code}"
        )
    metadata_json = json.loads(metadata_response.content)
    git_sha = metadata_json.get("git_sha", "")
    fingerprint = metadata_json.get("fingerprint", "")
    LOG.debug(f"git sha value - {git_sha}")
    LOG.debug(f"fingerprint value - {fingerprint}")
    return fingerprint, git_sha


def compare_squad_sha(squad_project_id, squad_project, current_sha_value) -> bool:
    if squad_project_id is None or squad_project is None:
        LOG.warning("\t** Previous SHA from SQUAD is not available")
        return True
    previous_sha_from_squad = squad_metadata_request(squad_project_id, squad_project)[1]
    if previous_sha_from_squad != current_sha_value:
        LOG.info(
            f"\t-> sha: {current_sha_value} vs \
        previous sha {previous_sha_from_squad}"
        )
        return True
    LOG.info(f"\t-> sha: {current_sha_value}")
    LOG.info("\t-> no changes")
    return False



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT


class TuxtriggerException(Exception):
    def __str__(self):
        name = super().__str__()
        if hasattr(self, "msg"):
            return self.msg.format(name=name)
        else:
            return name


class InvalidArgument(TuxtriggerException):
    pass


class TuxsuiteException(TuxtriggerException):
    pass


class SquadException(TuxtriggerException):
    pass



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT


from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry

import requests


TIMEOUT = 60


def get_session(retries):
    session = requests.Session()

    retry_strategy = Retry(
        total=retries,
        status_forcelist=[408, 413, 429, 500, 502, 503, 504],
        allowed_methods=["HEAD", "GET", "OPTIONS", "DELETE", "PUT", "TRACE"],
        backoff_factor=1,
    )

    adapter = HTTPAdapter(max_retries=retry_strategy)
    session.mount("http://", adapter)
    session.mount("https://", adapter)

    return session


def requests_get(*args, **kwargs):
    session = get_session(retries=10)
    return session.get(*args, timeout=TIMEOUT, **kwargs)


def requests_post(*args, **kwargs):
    session = get_session(retries=0)
    return session.post(*args, timeout=TIMEOUT, **kwargs)



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT
import argparse
from pathlib import Path
from tuxtrigger import __version__


OUTPUT_FILE = Path("share/gitsha.yaml")
PLAN_PATH = Path("share/plans/")
LOG_FILE = Path("log.txt")


def setup_parser() -> argparse.ArgumentParser:
    parser = argparse.ArgumentParser(
        prog="TuxTrigger",
        description="TuxTrigger command line tool for controlling changes in repositories",
    )
    parser.add_argument(
        "config", type=Path, help="config yaml file name", action="store"
    )
    parser.add_argument(
        "--sha-compare",
        type=str,
        help="choose compare method",
        default="squad",
        choices=["squad", "yaml"],
    )
    parser.add_argument(
        "--output",
        type=Path,
        help="path for storing output file",
        action="store",
        default=OUTPUT_FILE,
    )
    parser.add_argument(
        "--pre-submit",
        type=Path,
        help="pre-tuxsuite script to run",
        action="store",
    )
    parser.add_argument(
        "-v",
        "--version",
        help="prints current version",
        action="version",
        version=f"%(prog)s, {__version__}",
    )
    parser.add_argument(
        "--submit",
        "-s",
        type=str,
        help="trigger build on change",
        default="change",
        choices=["never", "change", "always"],
    )
    parser.add_argument(
        "--log-file",
        type=Path,
        help="print log to file",
        default=LOG_FILE,
        action="store",
    )
    parser.add_argument(
        "--plan",
        type=Path,
        help="path to plan files",
        default=PLAN_PATH,
        action="store",
    )
    parser.add_argument(
        "--log-level",
        "-l",
        type=str,
        help="set log level to more specific information",
        default="info",
        choices=["debug", "info", "warn", "error"],
    )
    parser.add_argument(
        "--json-out",
        type=Path,
        help="json output folder path",
        action="store",
    )
    parser.add_argument(
        "--plan-disabled",
        help="disable submiting plan to tuxsuite",
        action="store_false",
    )
    parser.add_argument(
        "--generate-config",
        help="dry-run to show generated config from regex value",
        action="store_true",
    )
    return parser



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import contextlib
import logging
from pathlib import Path
import sys
import logging.handlers
import time
import yaml

from tuxtrigger.argparser import setup_parser
from tuxtrigger.builder import (
    build_result,
    check_squad_project,
    compare_squad_sha,
    squad_metadata_request,
    tux_console_build,
    tux_console_plan,
    update_squad_metadata,
)
from tuxtrigger.yamlload import (
    pre_tux_run,
    yaml_file_read,
    create_repo_list,
    yaml_file_write,
    compare_sha,
    create_squad_project,
    run_lsremote,
    create_dynamic_configuration,
)
from tuxtrigger.manifest import (
    git_manifest_download,
    git_repository_fingerprint,
    manifest_changed,
)

LOG = logging.getLogger("tuxtrigger")

build_params = {
    "git_repo": "",
    "git_ref": "",
    "target_arch": "x86_64",
    "toolchain": "gcc-12",
    "kconfig": "tinyconfig",
}


def main() -> int:
    parser = setup_parser()
    options = parser.parse_args()
    plan_path = options.plan.absolute()
    pre_tux_script = None
    json_save = options.json_out
    if options.pre_submit:
        pre_tux_script = options.pre_submit.absolute()

    log_handler = logging.StreamHandler(sys.stdout)
    log_handler.setFormatter(logging.Formatter("%(message)s"))
    log_file_handler = logging.handlers.WatchedFileHandler(
        (options.log_file).absolute(),
        mode="w",
    )
    LOG.addHandler(log_handler)
    LOG.addHandler(log_file_handler)

    if options.log_level == "debug":
        LOG.setLevel(logging.DEBUG)
    elif options.log_level == "warn":
        LOG.setLevel(logging.WARNING)
    elif options.log_level == "error":
        LOG.setLevel(logging.ERROR)
    else:
        LOG.setLevel(logging.INFO)
    if options.json_out:
        json_save = Path(options.json_out)
    if options.config:
        generate_yaml_data = yaml_file_read(options.config)
        run_config = create_dynamic_configuration(generate_yaml_data)
        if options.generate_config:
            sys.stdout.write(
                yaml.dump(run_config, default_flow_style=False, sort_keys=False)
            )
            return 0
        uid_queue = list()
        output_data = dict()
        squad_config = dict()
        manifest_json = None
        archive_data_file = None
        repo_list = create_repo_list(run_config)
        if any(
            "git.kernel.org" in repo["url"].rstrip("/")
            for repo in run_config["repositories"]
        ):
            manifest_json = git_manifest_download()

        if options.sha_compare == "yaml":
            output_file = (options.output).absolute()
            output_file.parent.mkdir(exist_ok=True)
            with contextlib.suppress(FileNotFoundError):
                archive_data_file = yaml_file_read(output_file)
                previous_fingerprint = archive_data_file

        LOG.info("Checking repositories:")
        for repo in repo_list:
            value_dict = dict()
            repo_name = repo.url_path()
            group_name = repo.squad_group
            fingerprint = ""
            if "git.kernel.org" in repo_name.netloc:
                fingerprint = git_repository_fingerprint(manifest_json, repo_name.path)
            LOG.info(f"* {repo.url}")
            for branch in repo.branches:
                LOG.info(f' * branch: {branch["name"]}')
                repo_changed = False
                build_params["git_repo"] = repo.url
                build_params["git_ref"] = branch["name"]
                value_dict["fingerprint"] = fingerprint
                squad_project = branch.get(
                    "squad_project",
                    create_squad_project(repo.url_path(), branch["name"]),
                )
                project_id = check_squad_project(group_name, squad_project)

                if archive_data_file is None:
                    previous_fingerprint = squad_metadata_request(
                        project_id, squad_project
                    )[0]
                if (
                    fingerprint == ""
                    or options.submit == "always"
                    or manifest_changed(
                        repo_name.geturl(), fingerprint, previous_fingerprint
                    )
                ):
                    ls_remote_result = run_lsremote(repo.url, branch["name"])
                    value_dict.update(ls_remote_result)

                    if options.sha_compare == "squad":
                        repo_changed = compare_squad_sha(
                            project_id,
                            squad_project,
                            ls_remote_result[branch["name"]]["sha"],
                        )

                    elif options.sha_compare == "yaml":
                        repo_changed = compare_sha(
                            repo_name.geturl(),
                            branch["name"],
                            ls_remote_result,
                            archive_data_file,
                        )

                pre_tux_run(
                    pre_tux_script,
                    repo.url,
                    branch["name"],
                    value_dict,
                    repo_changed,
                )

                if options.submit == "never":
                    LOG.info("** Builds suspended **")

                elif repo_changed or options.submit == "always":
                    build_uid = tux_console_build(**build_params)
                    uid_queue.append(build_uid)
                    squad_config[build_uid] = (
                        branch.get("plan", ""),
                        group_name,
                        squad_project,
                        fingerprint,
                    )
            output_data[repo_name.geturl()] = value_dict

        LOG.info("** Build Phase Completed **")

        while uid_queue:
            for uid in uid_queue:
                time.sleep(10)
                json_build_output = build_result(uid)
                if json_build_output["git_describe"] is not None:
                    LOG.debug(f"\t*UID: {uid}")
                    LOG.debug(f"\t*Plan file name: {squad_config[uid][0]}")
                    LOG.debug(f"\t*SQUAD group name: {squad_config[uid][1]}")
                    LOG.debug(f"\t*SQUAD project name: {squad_config[uid][2]}")
                    LOG.debug(f"\t*Manifest Fingerprint: {squad_config[uid][3]}")
                    # when arg plan-disabled is inserted value of corresponding options.plan_disabled is changed to FALSE
                    if options.plan_disabled:
                        tux_console_plan(
                            json_build_output,
                            plan_path / squad_config[uid][0],
                            squad_config[uid][1],
                            squad_config[uid][2],
                            json_save,
                            squad_config[uid][3],
                        )
                    if options.sha_compare == "squad":
                        update_squad_metadata(
                            json_build_output,
                            squad_config[uid][1],
                            squad_config[uid][2],
                            squad_config[uid][3],
                        )
                    uid_queue.remove(uid)
        LOG.info("** Submiting Plans Phase Completed **")
        if options.sha_compare == "yaml":
            yaml_file_write(output_data, output_file)
            LOG.info("** SHA List Updated **")
    else:
        LOG.info("* Please input path to config yaml file!")
        parser.print_usage()

    return 0


def start() -> None:
    if __name__ == "__main__":
        sys.exit(main())


start()



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import json
import sys
import pytest
import yaml

from pathlib import Path
import tuxtrigger.__main__ as main_module

BASE = (Path(__file__) / "../..").resolve()

VALUE_DICT = {
    "master": {"sha": "12345", "ref": "refs/master/torvalds"},
    "v5.19": {"sha": "12345", "ref": "refs/v5.19"},
    "for-laurent": {"sha": "12345", "ref": "refs/for-laurent"},
}

UID = "2CCI3BkwKdqM4wOOwB5xRRxvOha"

JSON_OUTPUT = {
    "empty": "no_real_values",
    "uid": "1234",
    "git_repo": "https://gitlab.com/no_real_repo",
    "git_ref": "master",
    "git_describe": "v5.19-rc7",
}

MANIFEST_JSON = {
    "/pub/scm/linux/kernel/git/tomba/linux.git": {
        "owner": "Tomi Valkeinen",
        "description": "Tomi Valkeinen's kernel tree",
        "modified": 1660133218,
        "reference": "/pub/scm/linux/kernel/git/paulg/4.8-rt-patches.git",
        "fingerprint": "8fa23329efa65477f077d99e145e4087190a55cc",
        "forkgroup": "af9f4487-d538-46e5-b148-e18dfb461f8a",
        "head": "ref: refs/heads/master",
    },
    "/pub/scm/linux/kernel/git/torvalds/linux.git": {
        "symlinks": ["/pub/scm/linux/kernel/git/torvalds/linux-2.6.git"],
        "description": "Linux kernel source tree",
        "reference": "/pub/scm/linux/kernel/git/paulg/4.8-rt-patches.git",
        "modified": 1661194011,
        "fingerprint": "2c8d80ee6d795dc6951bbdef466ef19c64ff717d",
        "owner": "Linus Torvalds",
        "head": "ref: refs/heads/master",
        "forkgroup": "af9f4487-d538-46e5-b148-e18dfb461f8a",
    },
}

CREATED_CONFIG = {
    "repositories": [
        {
            "url": "https://git.kernel.org/pub/scm/linux/kernel/git/tomba/linux.git",
            "squad_group": "~non.existing",
            "branches": [
                {
                    "name": "master",
                    "squad_project": "example_project",
                    "plan": "stable.yaml",
                },
                {
                    "name": "for-laurent",
                    "squad_project": "example_project",
                    "plan": "stable-next.yaml",
                },
            ],
        },
        {
            "url": "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
            "squad_group": "~non.existing",
            "branches": [
                {
                    "name": "master",
                    "squad_project": "example_project",
                    "plan": "stable.yaml",
                },
                {
                    "name": "v5.19",
                    "squad_project": "example_project",
                    "plan": "stable-next.yaml",
                },
            ],
        },
    ]
}


@pytest.fixture
def argv():
    return ["tuxtrigger"]


@pytest.fixture(autouse=True)
def patch_argv(monkeypatch, argv):
    monkeypatch.setattr(sys, "argv", argv)


class TestMain:
    def test_start(self, monkeypatch, mocker):
        monkeypatch.setattr(main_module, "__name__", "__main__")
        main = mocker.patch("tuxtrigger.__main__.main", return_value=1)
        exit = mocker.patch("sys.exit")
        main_module.start()
        main.assert_called()
        exit.assert_called_with(1)

    def test_main_version(self, argv, capsys):
        argv.append("-v")
        with pytest.raises(SystemExit):
            main_module.main()
        out, out_err = capsys.readouterr()
        assert "TuxTrigger" in out

    def test_main_correct_data(self, argv, monkeypatch, mocker):
        argv.extend([f"{BASE}/test/test_files/happy_path.yaml"])
        monkeypatch.setattr(main_module, "__name__", "__main__")
        main_manifest = mocker.patch(
            "tuxtrigger.__main__.git_manifest_download",
            return_value=json.dumps(MANIFEST_JSON),
        )
        main_create_dynamic_configuration = mocker.patch(
            "tuxtrigger.__main__.create_dynamic_configuration",
            return_value=CREATED_CONFIG,
        )
        main_manifest_changed = mocker.patch(
            "tuxtrigger.__main__.manifest_changed", return_value=True
        )
        main_check_squad_project = mocker.patch(
            "tuxtrigger.__main__.check_squad_project", return_value=666
        )
        main_lsremote = mocker.patch(
            "tuxtrigger.__main__.run_lsremote", return_value=VALUE_DICT
        )
        main_build_uid = mocker.patch(
            "tuxtrigger.__main__.tux_console_build", return_value=UID
        )
        main_build_output = mocker.patch(
            "tuxtrigger.__main__.build_result", return_value=JSON_OUTPUT
        )
        main_compare_squad_sha = mocker.patch(
            "tuxtrigger.__main__.compare_squad_sha", return_value=True
        )
        main_tux_console_plan = mocker.patch(
            "tuxtrigger.__main__.tux_console_plan", return_value=0
        )
        main_squad_metadata_request = mocker.patch(
            "tuxtrigger.__main__.squad_metadata_request", return_value=("", "")
        )
        main_tux_console_plan = mocker.patch(
            "tuxtrigger.__main__.tux_console_plan", return_value=0
        )
        main_update_squad_metadata = mocker.patch(
            "tuxtrigger.__main__.update_squad_metadata", return_value=0
        )
        exit = mocker.patch("sys.exit")
        main_module.start()
        main_manifest.assert_called()
        main_create_dynamic_configuration.assert_called()
        main_manifest_changed.assert_called()
        main_check_squad_project.assert_called()
        main_lsremote.assert_called()
        main_build_uid.assert_called()
        main_compare_squad_sha.assert_called()
        main_build_output.assert_called()
        main_squad_metadata_request.assert_called()
        main_tux_console_plan.assert_called()
        main_update_squad_metadata.assert_called_with(
            {
                "empty": "no_real_values",
                "uid": "1234",
                "git_repo": "https://gitlab.com/no_real_repo",
                "git_ref": "master",
                "git_describe": "v5.19-rc7",
            },
            "~non.existing",
            "example_project",
            "2c8d80ee6d795dc6951bbdef466ef19c64ff717d",
        )
        exit.assert_called_with(0)

    def test_main_correct_data_yaml(self, argv, monkeypatch, mocker):
        argv.extend([f"{BASE}/test/test_files/happy_path.yaml"])
        argv.append("--sha-compare=yaml")
        monkeypatch.setattr(main_module, "__name__", "__main__")
        main_create_dynamic_configuration = mocker.patch(
            "tuxtrigger.__main__.create_dynamic_configuration",
            return_value=CREATED_CONFIG,
        )
        main_manifest = mocker.patch(
            "tuxtrigger.__main__.git_manifest_download",
            return_value=json.dumps(MANIFEST_JSON),
        )
        main_manifest_changed = mocker.patch(
            "tuxtrigger.__main__.manifest_changed", return_value=True
        )
        main_check_squad_project = mocker.patch(
            "tuxtrigger.__main__.check_squad_project", return_value=666
        )
        main_lsremote = mocker.patch(
            "tuxtrigger.__main__.run_lsremote", return_value=VALUE_DICT
        )
        main_compare_sha = mocker.patch(
            "tuxtrigger.__main__.compare_sha", return_value=True
        )
        main_build_uid = mocker.patch(
            "tuxtrigger.__main__.tux_console_build", return_value=UID
        )
        main_build_output = mocker.patch(
            "tuxtrigger.__main__.build_result", return_value=JSON_OUTPUT
        )
        main_tux_console_plan = mocker.patch(
            "tuxtrigger.__main__.tux_console_plan", return_value=0
        )
        main_pre_tux_run = mocker.patch(
            "tuxtrigger.__main__.pre_tux_run", return_value=0
        )
        exit = mocker.patch("sys.exit")
        main_module.start()
        main_create_dynamic_configuration.assert_called()
        main_manifest.assert_called()
        main_manifest_changed.assert_called()
        main_check_squad_project.assert_called()
        main_lsremote.assert_called()
        main_compare_sha.assert_called()
        main_pre_tux_run.assert_called()
        main_build_uid.assert_called()
        main_build_output.assert_called()
        main_tux_console_plan.assert_called()
        exit.assert_called_with(0)

    def test_main_key_error(self, argv):
        argv.extend([f"{BASE}/test/test_files/error_path.yaml"])
        with pytest.raises(Exception):
            main_module.main()
            assert main_module.main() == 1

    def test_main_build_never(self, argv, capsys, monkeypatch, mocker):
        argv.append(f"{BASE}/test/test_files/happy_path.yaml")
        monkeypatch.setattr(main_module, "__name__", "__main__")
        main = mocker.patch("tuxtrigger.__main__.run_lsremote", return_value=VALUE_DICT)
        argv.append("--submit=never")
        argv.append("--sha-compare=yaml")
        main_create_dynamic_configuration = mocker.patch(
            "tuxtrigger.__main__.create_dynamic_configuration",
            return_value=CREATED_CONFIG,
        )
        main_manifest = mocker.patch(
            "tuxtrigger.__main__.git_manifest_download",
            return_value=json.dumps(MANIFEST_JSON),
        )
        main_manifest_changed = mocker.patch(
            "tuxtrigger.__main__.manifest_changed", return_value=True
        )
        main_check_squad_project = mocker.patch(
            "tuxtrigger.__main__.check_squad_project", return_value=666
        )
        main_module.main()
        out, out_err = capsys.readouterr()
        main_create_dynamic_configuration.assert_called()
        main_manifest.assert_called()
        main_check_squad_project.assert_called()
        main_manifest_changed.assert_called()
        main.assert_called()
        assert "** Builds suspended **" in out

    def test_main_build_always(self, argv, mocker, monkeypatch):
        argv.append(f"{BASE}/test/test_files/happy_path.yaml")
        argv.append("--submit=always")
        argv.append("--sha-compare=yaml")
        monkeypatch.setattr(main_module, "__name__", "__main__")
        main_create_dynamic_configuration = mocker.patch(
            "tuxtrigger.__main__.create_dynamic_configuration",
            return_value=CREATED_CONFIG,
        )
        main_manifest = mocker.patch(
            "tuxtrigger.__main__.git_manifest_download",
            return_value=json.dumps(MANIFEST_JSON),
        )
        main_lsremote = mocker.patch(
            "tuxtrigger.__main__.run_lsremote", return_value=VALUE_DICT
        )
        main_build_uid = mocker.patch(
            "tuxtrigger.__main__.tux_console_build", return_value=UID
        )
        main_build_output = mocker.patch(
            "tuxtrigger.__main__.build_result", return_value=JSON_OUTPUT
        )
        main_tux_console_plan = mocker.patch(
            "tuxtrigger.__main__.tux_console_plan", return_value=0
        )
        main_pre_tux_run = mocker.patch(
            "tuxtrigger.__main__.pre_tux_run", return_value=0
        )
        main_check_squad_project = mocker.patch(
            "tuxtrigger.__main__.check_squad_project", return_value=666
        )
        main_module.main()
        main_create_dynamic_configuration.assert_called()
        main_manifest.assert_called()
        main_check_squad_project.assert_called()
        main_lsremote.assert_called()
        main_pre_tux_run.assert_called()
        main_build_uid.assert_called()
        main_build_output.assert_called()
        main_tux_console_plan.assert_called()

    def test_main_generate_config(self, argv, capsys, mocker, monkeypatch):
        argv.append(f"{BASE}/test/test_files/happy_path.yaml")
        argv.append("--generate-config")
        monkeypatch.setattr(main_module, "__name__", "__main__")
        main_generate_config = mocker.patch(
            "tuxtrigger.__main__.create_dynamic_configuration",
            return_value=CREATED_CONFIG,
        )
        main_module.main()
        main_generate_config.assert_called()
        stdout, stderr = capsys.readouterr()
        assert (
            yaml.dump(CREATED_CONFIG, default_flow_style=False, sort_keys=False)
            in stdout
        )



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from pathlib import Path
from unittest import mock
import pytest
import json

from tuxtrigger.exceptions import TuxtriggerException
from tuxtrigger.manifest import (
    git_manifest_download,
    git_repository_fingerprint,
    manifest_changed,
)

BASE = (Path(__file__) / "..").resolve()

FINGERPRINT = "c8f105f060651674a38b2103190f2ca52381ce76"

REPO_URL = "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git"
MANIFEST_REPO = "/pub/scm/linux/kernel/git/wwguy/iwlwifi.git"


class RequestMock:
    def __init__(self, status_code, content):
        self.status_code = status_code
        self.content = content


@mock.patch("tuxtrigger.manifest.requests_get")
def test_git_manifest_download(mock_req):
    with open(BASE / "test_files/manifest.js.gz", "rb") as test_file:
        manifest = RequestMock(status_code=200, content=test_file.read())
        mock_req.return_value = manifest
        output = git_manifest_download()
    assert isinstance(json.loads(output), dict)


@mock.patch("tuxtrigger.manifest.requests_get")
def test_git_manifest_not_found(mock_req):
    manifest = RequestMock(status_code=404, content=None)
    mock_req.return_value = manifest
    with pytest.raises(TuxtriggerException) as ex:
        git_manifest_download()
    assert "TuxTrigger not able to download manifest 404" in str(ex.value)


@mock.patch("tuxtrigger.manifest.requests_get")
def test_git_repository_fingerprint(mock_manifest):
    with open(BASE / "test_files/manifest.js.gz", "rb") as test_file:
        manifest = RequestMock(status_code=200, content=test_file.read())
        mock_manifest.return_value = manifest
        output = git_manifest_download()
    test_fingerprint = git_repository_fingerprint(output, MANIFEST_REPO)
    assert test_fingerprint == FINGERPRINT
    with pytest.raises(KeyError) as exc:
        git_repository_fingerprint(output, "REPO_URL")
    assert "Tracked Repository does not exist in manifest" in str(exc)


def test_manifest_changed_true(correct_archive_read):
    assert manifest_changed(REPO_URL, FINGERPRINT, correct_archive_read) is True


def test_manifest_changed_false(
    wrong_archive_read, correct_archive_read, correct_archive_read_no_fingerprint
):
    assert manifest_changed(REPO_URL, FINGERPRINT, wrong_archive_read) is False

    with pytest.raises(KeyError):
        manifest_changed("", FINGERPRINT, correct_archive_read)
        manifest_changed(REPO_URL, "", correct_archive_read)
    assert (
        manifest_changed(REPO_URL, FINGERPRINT, correct_archive_read_no_fingerprint)
        is False
    )
    assert manifest_changed(REPO_URL, FINGERPRINT, FINGERPRINT) is False



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT


import logging
from unittest import mock
import pytest
from test.test_builder import MockedStdout
from tuxtrigger import yamlload

from tuxtrigger.exceptions import TuxtriggerException
from tuxtrigger.inputvalidation import YamlValidator
from tuxtrigger.yamlload import (
    yaml_file_read,
    yaml_file_write,
    compare_sha,
    create_repo_list,
    create_dynamic_configuration,
)
from pathlib import Path


BASE = (Path(__file__) / "..").resolve()

ERROR_PATH = BASE / "./test_files/error_path.yaml"
HAPPY_PATH = BASE / "./test_files/happy_path.yaml"
FILE_TO_CREATE = BASE / "./test_files/new_file.yaml"
SCRIPT = BASE / ".test_files/test_script.sh"

VALUE_DICT = {
    "v5.19": {
        "sha": "2437f53721bcd154d50224acee23e7dbb8d8c622",
        "ref": "refs/tags/v5.19",
    }
}
VALUE = "v5.19"
RIGHT_KEY = "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git"
WRONG_KEY = "/linux-not-existing"

DYNAMIC_CONFIG = {
    "repositories": [
        {
            "url": "https://git.kernel.org/pub/scm/linux/kernel/git/tomba/linux.git",
            "squad_group": "~non.existing",
            "branches": [
                {
                    "name": "master",
                    "squad_project": "example_project",
                    "plan": "stable.yaml",
                },
                {
                    "name": "for-laurent",
                    "squad_project": "example_project",
                    "plan": "stable-next.yaml",
                },
            ],
        },
        {
            "url": "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
            "squad_group": "~non.existing",
            "regex": "for-next/*",
            "default_plan": "default_plan",
            "squad_project_prefix": "testing",
            "branches": [
                {
                    "name": "master",
                    "squad_project": "example_project",
                    "plan": "stable.yaml",
                },
                {
                    "name": "v5.19",
                    "squad_project": "example_project",
                    "plan": "stable-next.yaml",
                },
                {
                    "name": "heads/test-next",
                    "squad_project": "testing-linux-heads-test-next",
                    "plan": "default_plan",
                },
            ],
        },
    ]
}


def test_yaml_file_read():
    assert type(yaml_file_read(ERROR_PATH)) == dict


def test_create_repo_list(repo_setup_good):
    assert isinstance(next(repo_setup_good), YamlValidator) is True
    assert repo_setup_good is not None
    with pytest.raises(TuxtriggerException) as exc:
        next(create_repo_list(None))
    assert "Data input is none" in str(exc)


def test_yaml_file_write(tmpdir):
    test_input = yaml_file_read(ERROR_PATH)
    yaml_file_write(test_input, (tmpdir / "test.yaml"))
    assert (tmpdir / "test.yaml").exists()


def test_compare_sha_correct(correct_archive_read):
    with pytest.raises(KeyError):
        compare_sha(WRONG_KEY, VALUE, VALUE_DICT, correct_archive_read)
    assert compare_sha(RIGHT_KEY, VALUE, VALUE_DICT, correct_archive_read) is True


def test_compare_sha_wrong(wrong_archive_read, correct_archive_read):
    assert compare_sha(WRONG_KEY, VALUE, VALUE_DICT, wrong_archive_read) is True
    assert compare_sha(RIGHT_KEY, VALUE, VALUE_DICT, wrong_archive_read) is True
    assert (
        compare_sha(RIGHT_KEY, "non_existing_value", VALUE_DICT, correct_archive_read)
        is False
    )


@mock.patch("tuxtrigger.yamlload.subprocess.run")
def test_run_lsremote(mock_run):
    ls_remote = MockedStdout(
        returncode=0,
        stdout="0066f1b0e27556381402db3ff31f85d2a2265858        refs/heads/master",
    )
    mock_run.return_value = ls_remote
    assert yamlload.run_lsremote(
        "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git", "master"
    ) == {
        "master": {
            "ref": "refs/heads/master",
            "sha": "0066f1b0e27556381402db3ff31f85d2a2265858",
        }
    }
    ls_remote_error = MockedStdout(returncode=1, stdout="", stderr="Serious Error")
    mock_run.return_value = ls_remote_error
    assert yamlload.run_lsremote(
        "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git", "master"
    ) == {"master": {"sha": "", "ref": ""}}


def test_pre_tux_run_none(correct_archive_read, caplog):
    with caplog.at_level(logging.INFO):
        yamlload.pre_tux_run(
            None,
            "https://git.kernel.org/pub/scm/linux/kernel/git/torvalds/linux.git",
            "v5.19",
            VALUE_DICT,
            correct_archive_read,
        )

    assert "No script path provided" in caplog.text


@mock.patch("tuxtrigger.yamlload.subprocess.run")
def test_pre_tux_run(mock_run, caplog, correct_archive_read):
    ls_remote = MockedStdout(returncode=0, stdout="Success")
    mock_run.return_value = ls_remote
    with caplog.at_level(logging.INFO):
        yamlload.pre_tux_run(
            "dummy_path", RIGHT_KEY, VALUE, VALUE_DICT, correct_archive_read
        )
    assert "performed astonishingly wonderful" in caplog.text

    ls_remote_not_zero = MockedStdout(returncode=1, stdout="", stderr="Failure")
    mock_run.return_value = ls_remote_not_zero
    with pytest.raises(TuxtriggerException) as exc:
        yamlload.pre_tux_run(
            "dummy_path", RIGHT_KEY, VALUE, VALUE_DICT, correct_archive_read
        )
    assert "Script was not invoked properly" in str(exc)


@mock.patch("tuxtrigger.yamlload.subprocess.run")
def test_run_lsremote_regex(mock_run):
    mocked_run = MockedStdout(
        returncode=0,
        stdout="0066f1b0e27556381402db3ff31f85d2a2265858\trefs/heads/test-next",
    )
    mock_run.return_value = mocked_run
    assert yamlload.run_lsremote_regex("example_repository", "example_regex*") == {
        "test-next": "0066f1b0e27556381402db3ff31f85d2a2265858"
    }

    mocked_run = MockedStdout(
        returncode=0,
        stdout="0066f1b0e27556381402db3ff31f85d2a2265858\trefs/heads/for/test-next",
    )
    mock_run.return_value = mocked_run
    assert yamlload.run_lsremote_regex("example_repository", "example_regex*") == {
        "for/test-next": "0066f1b0e27556381402db3ff31f85d2a2265858"
    }
    failed_run = MockedStdout(returncode=1, stdout="", stderr="ls remote failure")
    mock_run.return_value = failed_run
    assert yamlload.run_lsremote_regex("example_repository", "example_regex*") == {}


def test_create_dynamic_configuration_none():
    with pytest.raises(TuxtriggerException) as exc:
        create_dynamic_configuration(None)
    assert "Not able to generate data - config file not read" in str(exc)


@mock.patch("tuxtrigger.yamlload.run_lsremote_regex")
def test_create_dynamic_configuration(mock_run):
    mock_run.return_value = {
        "heads/test-next": "0066f1b0e27556381402db3ff31f85d2a2265858"
    }
    assert (
        yamlload.create_dynamic_configuration(yaml_file_read(HAPPY_PATH))
        == DYNAMIC_CONFIG
    )



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT


import json
from requests import Session
from unittest import mock

import pytest

from pathlib import Path
from tuxtrigger.builder import (
    compare_squad_sha,
    squad_metadata_request,
    squad_submit,
    tux_console_build,
    tux_console_plan,
    build_result,
    update_squad_metadata,
)
from tuxtrigger.exceptions import SquadException, TuxtriggerException, TuxsuiteException

BASE = (Path(__file__) / "..").resolve()

PLAN = BASE / "test_files/planTest.yaml"
PLAN_FAIL = BASE / "test_files/planTestc.yaml"
JSON_PLAN_RESULT = BASE / "test_files/plan_result.json"
JSON_OUT = BASE / "test_files/"

UID = "2CCI3BkwKdqM4wOOwB5xRRxvOha"
FINGERPRINT = "8fa23329efa65477f077d99e145e4087190a55cc"

PARAMS = {
    "git_repo": "https://gitlab.com/Linaro/lkft/mirrors/stable/linux-stable-rc",
    "git_ref": "master",
    "target_arch": "x86_64",
    "toolchain": "gcc-12",
    "kconfig": "tinyconfig",
}
JSON_DATA = {
    "empty": "no_real_values",
    "uid": "1234",
    "git_repo": "https://gitlab.com/no_real_repo",
    "git_ref": "master",
    "git_sha": "6fae37b8a05e58b6103198e89d12dc0d6d472d92",
    "git_describe": "test-rc",
}
JSON_BUILD_DATA = {
    "state": "provisioning",
    "result": "None",
    "uid": "1234",
    "git_repo": "https://gitlab.com/no_real_repo",
    "git_ref": "master",
    "git_describe": "test-rc",
}

JSON_RESPONSE = """{
"count":1,
"git_sha":"1234",
"fingerprint":"8fa23329efa65477f077d99e145e4087190a55cc",
"results":[
{
"metadata":"https://www.example_metadata.pl",
"id":1234
}
]
}"""


class MockedStdout:
    def __init__(self, returncode, stdout, stderr=None) -> None:
        self.returncode = returncode
        self.stdout = stdout
        self.stderr = stderr


class MockedSession:
    def __init__(self, status_code, content, ok=True) -> None:
        self.status_code = status_code
        self.content = content
        self.ok = ok


@mock.patch("tuxtrigger.builder.subprocess.run")
@mock.patch("tempfile.NamedTemporaryFile")
def test_tux_console_build(mock_temp_file, mock_run):
    build = MockedStdout(returncode=0, stdout=JSON_DATA)
    mock_run.return_value = build
    with open(BASE / "test_files/test.json", "r") as temp_file:
        mock_temp_file.return_value.__enter__.return_value = temp_file
        result = tux_console_build(**PARAMS)
    assert result == JSON_DATA["uid"]

    build_error = MockedStdout(returncode=1, stdout="", stderr="Tuxsuite Failure")
    mock_run.return_value = build_error
    with pytest.raises(TuxsuiteException) as exc:
        tux_console_build(**PARAMS)
    assert "Tuxsuite not build repo" in str(exc)


@mock.patch(
    "tuxtrigger.builder.subprocess.run",
    side_effect=TuxsuiteException("*** Tuxsuite not build repo"),
)
def test_tux_console_build_error(mock_run):
    build = MockedStdout(returncode=1, stdout=None)
    mock_run.return_value = build
    with pytest.raises(TuxsuiteException) as ex:
        tux_console_build(**PARAMS)
    assert "Tuxsuite not build" in str(ex.value)


@mock.patch("tuxtrigger.builder.subprocess.run")
def test_tux_console_plan_error(mock_run, squad_group_good, squad_project_good):
    build = MockedStdout(returncode=1, stdout=JSON_DATA)
    mock_run.return_value = build
    with pytest.raises(TuxtriggerException) as exc_tuxtrigger:
        tux_console_plan(
            None, PLAN, squad_group_good, squad_project_good, JSON_OUT, FINGERPRINT
        )

    assert "*** Not able to submit plan" in str(exc_tuxtrigger)
    with pytest.raises(TuxsuiteException) as exc_tuxsuite:
        tux_console_plan(
            JSON_DATA, PLAN, squad_group_good, squad_project_good, JSON_OUT, FINGERPRINT
        )
    assert "Submiting Plan for example_project_test-rc failed" in str(exc_tuxsuite)


@mock.patch("tuxtrigger.builder.subprocess.run")
@mock.patch("tempfile.NamedTemporaryFile")
def test_tux_console_plan(
    mock_temp_file, mock_run, squad_group_good, squad_project_good
):
    build = MockedStdout(returncode=0, stdout=JSON_DATA)
    mock_run.return_value = build
    with open(BASE / "test_files/test.json", "r") as test_file:
        mock_temp_file.return_value.__enter__.return_value = test_file
        fail_plan = tux_console_plan(
            JSON_DATA,
            PLAN_FAIL,
            squad_group_good,
            squad_project_good,
            JSON_OUT,
            FINGERPRINT,
        )
    mock_run.assert_called_with(
        [
            "squad-client",
            "submit-tuxsuite",
            "--group",
            "~non.existing",
            "--project",
            "example_project",
            "--build",
            "test-rc",
            "--backend",
            "tuxsuite.com",
            "--json",
            str(BASE / "test_files/test.json"),
        ],
        stdout=-1,
        stderr=-1,
        text=True,
    )
    assert fail_plan == 0


@mock.patch("tuxtrigger.builder.subprocess.run")
def test_build_result(mock_run):
    build = MockedStdout(returncode=0, stdout=json.dumps(JSON_BUILD_DATA))
    mock_run.return_value = build
    json_output = build_result(UID)
    mock_run.assert_called_once_with(
        ["tuxsuite", "build", "get", UID, "--json"], stdout=-1, stderr=-1, text=True
    )
    assert build_result(None) is None
    assert JSON_BUILD_DATA == json_output


@mock.patch("tuxtrigger.builder.subprocess.run")
def test_build_result_error(mock_run):
    build = MockedStdout(returncode=1, stdout="Oops!")
    mock_run.return_value = build
    with pytest.raises(TuxsuiteException) as exc:
        build_result(UID)
    assert "*** Build result for UID:2CCI3BkwKdqM4wOOwB5xRRxvOha failed" in str(exc)


def test_update_squad_metadata_error(squad_project_good):
    with pytest.raises(TuxtriggerException) as exc:
        update_squad_metadata(JSON_DATA, None, squad_project_good, FINGERPRINT)
    assert "SQUAD config is not available!" in str(exc)


@mock.patch.object(Session, "post")
def test_update_squad_metadata(mock_session, squad_group_good, squad_project_good):
    session_ok = MockedSession(status_code=200, content=JSON_RESPONSE, ok=True)
    mock_session.return_value = session_ok
    update_result = update_squad_metadata(
        JSON_DATA, squad_group_good, squad_project_good, FINGERPRINT
    )
    assert update_result == 0


@mock.patch("tuxtrigger.builder.subprocess.run")
def test_squad_submit(mock_run, squad_env_setup, squad_group_good, squad_project_good):
    mocked_submit = MockedStdout(
        returncode=1, stdout="Ooops!, not working!", stderr="Ooops! Serious Error!"
    )
    mock_run.return_value = mocked_submit
    with pytest.raises(SquadException) as exc:
        squad_submit(
            JSON_DATA,
            squad_group_good,
            squad_project_good,
            JSON_PLAN_RESULT,
            FINGERPRINT,
        )
    assert "*** squad-client not able to pass data" in str(exc)
    with pytest.raises(TuxtriggerException) as exc:
        squad_submit(JSON_DATA, None, squad_project_good, JSON_PLAN_RESULT, FINGERPRINT)
    assert "SQUAD config is not available! Unable to process" in str(exc)


@mock.patch.object(Session, "get")
def test_squad_sha_request(mock_session):
    session_ok = MockedSession(status_code=200, content=JSON_RESPONSE)
    mock_session.return_value = session_ok
    squad_sha = squad_metadata_request("1111", "tuxtrigger")
    assert squad_sha == ("8fa23329efa65477f077d99e145e4087190a55cc", "1234")


@mock.patch.object(Session, "get")
def test_squad_sha_request_fail(mock_session):
    session_fail = MockedSession(status_code=404, content=(None, None))
    mock_session.return_value = session_fail
    with pytest.raises(SquadException) as exc:
        squad_metadata_request("1112", "tuxtrigger")
    assert "SQUAD response error - (get latest build) 404" in str(exc)

    assert squad_metadata_request(None, None) == (None, None)


@mock.patch("tuxtrigger.builder.squad_metadata_request")
def test_compare_squad_sha(mock_sha):
    mock_sha.return_value = ("1234", "1234")
    assert compare_squad_sha("1111", "tuxtrigger", "1234") is False
    assert compare_squad_sha("1112", "tuxtrigger", "5678") is True


def test_compare_squad_sha_none():
    assert compare_squad_sha(None, None, "1234") is True



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT


from http.client import HTTPMessage
import json
import os
from unittest import mock
from unittest.mock import Mock, ANY, call
from tuxtrigger import request


class MockedRequest:
    def __init__(self, status_code):
        self.status_code = status_code


@mock.patch("urllib3.connectionpool.HTTPConnectionPool._get_conn")
def test_get_session(connection_mock):
    connection_mock.return_value.getresponse.side_effect = [
        Mock(status=500, msg=HTTPMessage()),
        Mock(status=429, msg=HTTPMessage()),
        Mock(status=200, msg=HTTPMessage()),
    ]
    r = request.get_session(3)
    response = r.get(url="http://anyurl.pl/testme")
    response.raise_for_status()
    assert connection_mock.return_value.request.mock_calls == [
        call("GET", "/testme", body=None, headers=ANY),
        call("GET", "/testme", body=None, headers=ANY),
        call("GET", "/testme", body=None, headers=ANY),
    ]


@mock.patch("tuxtrigger.request.requests_get")
def test_requests_get(mock_session):
    mock_req = MockedRequest(status_code=200)
    mock_session.return_value = mock_req
    req = request.requests_get(url="https://example.pl/xyz")
    assert req.status_code == 200


@mock.patch("tuxtrigger.request.requests_post")
def test_requests_post(mock_session):
    test_data = """{
        "fingerprint":"8fa23329efa65477f077d99e145e4087190a55cc",
        "git_sha":"5f9df76887bf8170e8844f1907c13fbbb30e9c36"
    }"""
    headers = {"Auth-Token": os.getenv("SQUAD_TOKEN")}
    mock_req = MockedRequest(status_code=201)
    mock_session.return_value = mock_req
    req = request.requests_post(
        url="https://example.pl/send-me-data",
        data=json.loads(test_data),
        headers=headers,
    )
    mock_session.assert_called_with(
        url="https://example.pl/send-me-data",
        data=json.loads(test_data),
        headers=headers,
    )
    assert req.status_code == 201



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import subprocess

from tuxtrigger import __version__
from pathlib import Path

BASE = (Path(__file__) / "../..").resolve()


def test_setup_parser():
    version_result = subprocess.run(
        ["sh", f"{BASE}/run", "-v"], stdout=subprocess.PIPE, text=True
    )
    help_result = subprocess.run(
        ["sh", f"{BASE}/run", "--help"], stdout=subprocess.PIPE, text=True
    )
    assert f"TuxTrigger, {__version__}" in version_result.stdout
    assert (
        "usage: TuxTrigger [-h] [--sha-compare {squad,yaml}] [--output OUTPUT]"
        in help_result.stdout
    )



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['tuxtrigger']

package_data = \
{'': ['*']}

install_requires = \
['tuxsuite', 'squad-client', 'jinja2', 'requests', 'PyYAML']

entry_points = \
{'console_scripts': ['tuxtrigger = tuxtrigger.__main__:main']}

setup(name='tuxtrigger',
      version='0.6.5',
      description='TuxTrigger Project by Linaro',
      author='Pawel Szymaszek',
      author_email='pawel.szymaszek@linaro.org',
      url='https://tuxsuite.com/',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      entry_points=entry_points,
      python_requires='>=3.6',
     )



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

import json
import logging
from pathlib import Path
import tempfile
import gzip

from tuxtrigger.request import requests_get
from tuxtrigger.exceptions import TuxtriggerException

LOG = logging.getLogger("tuxtrigger")


def git_manifest_download() -> bytes:
    url = "https://git.kernel.org/manifest.js.gz"
    manifest_request = requests_get(url)

    if manifest_request.status_code != 200:
        LOG.warning(f"*manifest error code {manifest_request.status_code}")
        raise TuxtriggerException(
            f"*** TuxTrigger not able to download manifest {manifest_request.status_code}"
        )

    LOG.debug(f"*manifest response {manifest_request.status_code}")

    with tempfile.TemporaryDirectory() as manifest_temp:
        Path(f"{manifest_temp}/manifest.js.gz").write_bytes(manifest_request.content)

        with gzip.open(f"{manifest_temp}/manifest.js.gz", "rb") as gz_file:
            json_output = gz_file.read()
        LOG.debug(json_output)

    return json_output


def git_repository_fingerprint(json_data, repo_url):
    json_parser = json.loads(json_data)
    if repo_url not in json_parser:
        raise KeyError(f"Tracked Repository does not exist in manifest {repo_url}")
    return json_parser[repo_url]["fingerprint"]


def manifest_changed(repo_url, fingerprint, archive_data) -> bool:
    if archive_data is None:
        LOG.warning("\t*** Data Input is none, not able to compare fingerprint")
        LOG.debug(f"\t-> fingerprint: {fingerprint}")
        return False
    if isinstance(archive_data, dict) and "fingerprint" not in archive_data[repo_url]:
        LOG.warning("\t*** Fingerprint not found in yaml file")
        LOG.debug(f"\t*** Repo name: {repo_url}")
        return False
    if isinstance(archive_data, dict):
        old_fingerprint = archive_data[repo_url]["fingerprint"]
    if isinstance(archive_data, str):
        old_fingerprint = archive_data
    if not fingerprint == old_fingerprint:
        LOG.info(
            f"\t-> fingerprint: {fingerprint} vs \
        previous fingerprint {old_fingerprint}"
        )
        return True
    LOG.info(f"\t-> fingerprint: {fingerprint}")
    LOG.info("\t-> no changes")
    return False



#
# Copyright 2022-present Linaro Limited
#
# SPDX-License-Identifier: MIT

from tuxtrigger.exceptions import (
    InvalidArgument,
    SquadException,
    TuxtriggerException,
    TuxsuiteException,
)


def test_tux_trig_exception():
    exc = TuxtriggerException("test message")
    assert isinstance(exc, Exception) is True
    assert exc.__str__() == "test message"


def test_inheritance():
    arg_exc = InvalidArgument()
    tux_exc = TuxsuiteException()
    squad_exc = SquadException()
    assert isinstance(arg_exc, TuxtriggerException) is True
    assert isinstance(tux_exc, TuxtriggerException) is True
    assert isinstance(squad_exc, TuxtriggerException) is True
